@@ -57,7 +57,6 @@ __maintainer__ = '''Costas Tyfoxylos'''
 __email__ = '''<ctyfoxylos@schubergphilis.com>'''
 __status__ = '''Development'''  # "Prototype", "Development", "Production".
 
-
 LOGGER_BASENAME = '''entities'''
 LOGGER = logging.getLogger(LOGGER_BASENAME)
 LOGGER.addHandler(logging.NullHandler())
@@ -96,6 +95,16 @@ class LandingZone:
     def __repr__(self):
         return f'{self.name} landing zone'
 
+    @property
+    def account_ids(self):
+        """Accounts ids of the accounts.
+
+        Returns:
+            List of account ids for provided accounts
+
+        """
+        return [account.id for account in self.accounts]
+
     @property
     @cached(cache=TTLCache(maxsize=1000, ttl=600))
     def accounts(self):
@@ -146,13 +155,17 @@ class LandingZone:
 
 
 @dataclass
-class AwsAccount:
+class AwsAccount:  # pylint: disable=too-many-instance-attributes
     """Models the aws account that can label itself."""
 
     id: str  # pylint: disable=invalid-name
     name: str
     landing_zone: LandingZone
     energy_label: str = ""
+    number_of_critical_high_findings: int = 0
+    number_of_medium_findings: int = 0
+    number_of_low_findings: int = 0
+    max_days_open: int = 0
 
     def __post_init__(self):
         self._logger = logging.getLogger(f'{LOGGER_BASENAME}.{self.__class__.__name__}')
@@ -174,30 +187,39 @@ class AwsAccount:
                 open_findings = df[(df['Account ID'] == self.id) & (df['Workflow State'] != 'RESOLVED')]
                 number_of_critical_findings = open_findings[open_findings['Severity'] == 'CRITICAL'].shape[0]
                 number_of_high_findings = open_findings[open_findings['Severity'] == 'HIGH'].shape[0]
-                number_of_medium_findings = open_findings[open_findings['Severity'] == 'MEDIUM'].shape[0]
-                number_of_low_findings = open_findings[open_findings['Severity'] == 'LOW'].shape[0]
+                self.number_of_critical_high_findings = number_of_critical_findings + number_of_high_findings
+                self.number_of_medium_findings = open_findings[open_findings['Severity'] == 'MEDIUM'].shape[0]
+                self.number_of_low_findings = open_findings[open_findings['Severity'] == 'LOW'].shape[0]
                 open_findings_low_or_higher = open_findings[(open_findings['Severity'] == 'LOW') |
                                                             (open_findings['Severity'] == 'MEDIUM') |
                                                             (open_findings['Severity'] == 'HIGH') |
                                                             (open_findings['Severity'] == 'CRITICAL')]
-                nr_open_findings_low_or_higher = open_findings_low_or_higher.shape[0]
+                self.max_days_open = max(open_findings_low_or_higher['Days Open']) \
+                    if open_findings_low_or_higher['Days Open'].shape[0] > 0 else 0
+
+                self._logger.debug(f'Calculating for account {self.id} '
+                                   f'with number of critical+high findings '
+                                   f'{self.number_of_critical_high_findings}, '
+                                   f'number of medium findings {self.number_of_medium_findings}, '
+                                   f'number of low findings {self.number_of_low_findings}, '
+                                   f'and findings have been open for over '
+                                   f'{self.max_days_open} days')
                 for threshold in self.landing_zone.account_thresholds:
-                    nr_of_findings_open_lt_xx_days = \
-                        open_findings_low_or_higher[
-                            open_findings_low_or_higher['Days Open'] < threshold['days_open_less_than']].shape[0]
-                    if (number_of_critical_findings + number_of_high_findings) <= threshold['critical_high'] \
-                            and number_of_medium_findings <= threshold['medium'] \
-                            and number_of_low_findings <= threshold['low'] \
-                            and nr_open_findings_low_or_higher <= nr_of_findings_open_lt_xx_days:
+                    if self.number_of_critical_high_findings <= threshold['critical_high'] \
+                            and self.number_of_medium_findings <= threshold['medium'] \
+                            and self.number_of_low_findings <= threshold['low'] \
+                            and self.max_days_open < threshold['days_open_less_than']:
                         self.energy_label = threshold['label']
+                        self._logger.debug(f'Energy Label for account {self.id} '
+                                           f'has been calculated: {self.energy_label}')
                         break
-            except Exception:   # pylint: disable=broad-except
+            except Exception:  # pylint: disable=broad-except
                 self._logger.exception(f'Could not calculate energy label for account {self.id}, using the default "F"')
         return self.energy_label
 
 
-@dataclass  # pylint: disable=too-many-public-methods
-class Finding:
+@dataclass
+class Finding:  # pylint: disable=too-many-public-methods
     """Models a finding."""
 
     _data: dict
@@ -354,19 +376,21 @@ class SecurityHub:  # pylint: disable=too-few-public-methods
 
     instance = None
 
-    def __new__(cls, query_filter, region=None):
+    def __new__(cls, query_filter, region=None, allowed_regions=None, denied_regions=None):
         if not SecurityHub.instance:
-            SecurityHub.instance = _SecurityHub(query_filter, region)
+            SecurityHub.instance = _SecurityHub(query_filter, region, allowed_regions, denied_regions)
         return SecurityHub.instance
 
 
-class _SecurityHub:
+class _SecurityHub:  # pylint: disable=too-many-instance-attributes
     """Models security hub and can retrieve findings."""
 
     frameworks = {'cis', 'pci-dss', 'aws-foundational-security-best-practices'}
 
-    def __init__(self, query_filter, region=None):
+    def __init__(self, query_filter, region=None, allowed_regions=None, denied_regions=None):
         self._logger = logging.getLogger(f'{LOGGER_BASENAME}.{self.__class__.__name__}')
+        self.allowed_regions = allowed_regions
+        self.denied_regions = denied_regions
         self.sts = boto3.client('sts')
         self.ec2 = self._get_client(region)
         self.query_filter = query_filter
@@ -397,7 +421,17 @@ class _SecurityHub:
             self._aws_regions = [region.get('RegionName')
                                  for region in self.ec2.describe_regions().get('Regions')
                                  if not region.get('OptInStatus', '') == 'not-opted-in']
-            self._logger.debug(f'Regions that were opted in are : {self._aws_regions}')
+            self._logger.debug(f'Regions in EC2 that were opted in are : {self._aws_regions}')
+
+        if self.allowed_regions:
+            self._aws_regions = set(self._aws_regions).intersection(set(self.allowed_regions))
+            self._logger.debug(f'Working on allowed regions {self._aws_regions}')
+        elif self.denied_regions:
+            self._logger.debug(f'Excluding denied regions {self.denied_regions}')
+            self._aws_regions = set(self._aws_regions) - set(self.denied_regions)
+            self._logger.debug(f'Working on non-denied regions {self._aws_regions}')
+        else:
+            self._logger.debug('Working on all regions')
         return self._aws_regions
 
     @property
@@ -419,7 +453,7 @@ class _SecurityHub:
                         finding = Finding(finding_data)
                         self._logger.debug(f'Adding finding with id {finding.id}')
                         findings.append(finding)
-            except security_hub.exceptions.InvalidAccessException:
+            except (security_hub.exceptions.InvalidAccessException, security_hub.exceptions.AccessDeniedException):
                 self._logger.warning(f'Check your access for Security Hub for region {region}.')
                 continue
         return findings
